import z from 'zod';
import * as zod_to_json_schema from 'zod-to-json-schema';
import OpenAI from 'openai';
import React from 'react';

type ValidZodSchema = z.ZodString | z.ZodNumber | z.AnyZodObject;
type TActionMetadata = ValidZodSchema | undefined;
type TActionFunctionExtras = ValidZodSchema | undefined;
type TActionInput = z.ZodObject<any>;
type TActionOutput = ValidZodSchema | z.ZodVoid;
type TTokenAuthMetadata = z.ZodObject<any> | undefined;
type TOAuthMetadata = z.ZodObject<any> | undefined;
type TActionType = "CLIENT" | "SERVER" | "ECHO";
type TActionFunction<TRegistry extends TAnyRegistryData, TInput extends TActionInput, TOutput extends TActionOutput, TAuth extends TAnyActionAuth> = (_: {
    input: z.output<TInput>;
    auth: TAuthArg<TAuth>;
    extras: TRegistry["actionFunctionExtrasSchema"] extends infer U ? U extends ValidZodSchema ? z.output<U> : undefined : undefined;
}) => Promise<z.input<TOutput>>;
type TRegistryData<TNamespace extends string, TMetadata extends TActionMetadata, TExtras extends TActionFunctionExtras, TToken extends TTokenAuthMetadata, TOAuth extends TOAuthMetadata> = {
    /**
     * The namespace for the actions registry. Useful if you have multiple action registries.
     *
     * @example "ADE"
     */
    namespace: TNamespace;
    /**
     * The schema for the metadata of the action. This is used to validate the metadata passed to the action.
     *
     * If you don't have metadata, you can pass `undefined`.
     *
     * @example
     * ```typescript
     * z.object({
     *   title: z.string(),
     *   description: z.string(),
     * })
     * ```
     */
    metadataSchema?: TMetadata;
    /**
     * Sometimes you may want to pass extra data to each action function (on top of input and auth). This is useful for things like user data, local time zone, etc.
     *
     * If you don't have any extra data, you can pass `undefined`.
     *
     * @example
     * ```typescript
     * z.object({
     *  userData: z.object({
     *   email: z.string().email(),
     *   name: z.string(),
     *  })
     * })
     * ```
     */
    actionFunctionExtrasSchema?: TExtras;
    /**
     * The schema for the token auth metadata. Useful if you want to store some extra data with the token.
     *
     * ```typescript
     * z.object({
     *  tokenGenerationDocumentationURL: z.string().url(),
     * })
     */
    tokenAuthMetadataSchema?: TToken;
    /**
     * The schema for the OAuth metadata. Useful if you want to store some extra data with the OAuth token.
     *
     * ```typescript
     * z.object({
     *   oauthAppGenerationDocumentationURL: z.string().url(),
     * })
     */
    oAuthMetadataSchema?: TOAuth;
};
type TAnyRegistryData = TRegistryData<string, TActionMetadata, TActionFunctionExtras, TTokenAuthMetadata, TOAuthMetadata>;
type TActionData<TRegistry extends TAnyRegistryData, TId extends string, TFunctionName extends string, TInput extends TActionInput, TOutput extends TActionOutput, TAuth extends TAnyActionAuth, TType extends TActionType> = {
    registryData: TRegistry;
    id: TId;
    actionType: TType;
    functionName: TFunctionName;
    metadata: TRegistry["metadataSchema"] extends infer U ? U extends ValidZodSchema ? z.output<U> : undefined : undefined;
    inputSchema: TInput;
    outputSchema: TOutput;
    authConfig: TAuth;
    actionFunction: TActionFunction<TRegistry, TInput, TOutput, TAuth>;
    exampleInput: z.input<TInput> | null;
};
type TAnyActionData = TActionData<any, any, any, any, any, any, any>;

type ValuesOf<T> = T[keyof T];
type FilterObjectByValue<T, U> = {
    [K in keyof T]: T[K] extends U ? K : never;
};

/**
 * The type of authentication.
 */
declare const AuthType: {
    readonly NONE: "None";
    readonly TOKEN: "Token";
    readonly OAUTH: "OAuth";
};
type TAuthType = ValuesOf<typeof AuthType>;
type TAuthInputMetadata<TRegistry extends TAnyRegistryData, K extends "tokenAuthMetadataSchema" | "oAuthMetadataSchema"> = TRegistry[K] extends infer U ? U extends z.AnyZodObject ? z.input<U> : {} : {};
type TAuthOutputMetadata<TRegistry extends TAnyRegistryData, K extends "tokenAuthMetadataSchema" | "oAuthMetadataSchema"> = TRegistry[K] extends infer U ? U extends z.AnyZodObject ? z.output<U> : {} : {};
declare const OAuthConfigSchema: z.ZodUnion<[z.ZodObject<{
    scopes: z.ZodArray<z.ZodString, "many">;
    discoveryEndpoint: z.ZodString;
}, "strip", z.ZodTypeAny, {
    scopes: string[];
    discoveryEndpoint: string;
}, {
    scopes: string[];
    discoveryEndpoint: string;
}>, z.ZodObject<{
    scopes: z.ZodArray<z.ZodString, "many">;
    discoveryEndpoint: z.ZodUndefined;
    authorizationEndpoint: z.ZodString;
    tokenEndpoint: z.ZodString;
    refreshEndpoint: z.ZodOptional<z.ZodString>;
    revokeEndpoint: z.ZodOptional<z.ZodString>;
    codeChallengeMethod: z.ZodNullable<z.ZodLiteral<"S256">>;
}, "strip", z.ZodTypeAny, {
    scopes: string[];
    authorizationEndpoint: string;
    tokenEndpoint: string;
    codeChallengeMethod: "S256" | null;
    discoveryEndpoint?: undefined;
    refreshEndpoint?: string | undefined;
    revokeEndpoint?: string | undefined;
}, {
    scopes: string[];
    authorizationEndpoint: string;
    tokenEndpoint: string;
    codeChallengeMethod: "S256" | null;
    discoveryEndpoint?: undefined;
    refreshEndpoint?: string | undefined;
    revokeEndpoint?: string | undefined;
}>]>;
type TOAuthConfig = z.infer<typeof OAuthConfigSchema>;
type TOAuthConfigWithOutputMetadata<TRegistry extends TAnyRegistryData> = TOAuthConfig & TAuthOutputMetadata<TRegistry, "oAuthMetadataSchema">;
type TOAuthConfigWithInputMetadata<TRegistry extends TAnyRegistryData> = TOAuthConfig & TAuthInputMetadata<TRegistry, "oAuthMetadataSchema">;
type TTokenCustomData = z.ZodObject<any> | null;
/**
 * Configuration for authenticating with an access token that users must generate themselves.
 *
 * This is useful for resources that don't support OAuth, but do support API keys.
 */
type TTokenAuthConfig<TCustomData extends TTokenCustomData> = {
    /**
     * Optional schema for custom data to be stored in the user's account that is needed to use the action.
     *
     * @example
     * For the Canvas LTS action, we also need to collect the user's Canvas domain (e.g. `https://canvas.calpoly.edu`).
     * ```
     * z.object({
     *  canvasDomain: z.string().url().refine((url) => url.startsWith("https://"), {
     *   message: "Canvas domain must start with https://",
     *  }),
     * })
     * ```
     */
    customDataSchema: TCustomData;
    /**
     * An optional function to validate the access token and custom data upon submission.
     *
     * @example
     * For the Canvas LTS action, we need to validate the access token and custom data by making a request to the Canvas API.
     *
     * ```typescript
     * validateToken: async ({ auth }) => {
     *  const response = await fetch(`${auth.customData.canvasDomain}/api/v1/users/self`, {
     *   headers: {
     *    Authorization: `Bearer ${auth.accessToken}`,
     *   },
     *  })
     *
     *  return { isValid: response.ok }
     * }
     * ```
     */
    validateToken?: (_: {
        auth: {
            customData: TCustomData extends z.ZodObject<any> ? z.infer<TCustomData> : null;
            accessToken: string;
        };
    }) => Promise<{
        isValid: boolean;
    }>;
};
type TTokenAuthConfigWithInputMetadata<TCustomData extends TTokenCustomData, TRegistry extends TAnyRegistryData> = TTokenAuthConfig<TCustomData> & TAuthInputMetadata<TRegistry, "tokenAuthMetadataSchema">;
type TTokenAuthConfigWithOutputMetadata<TCustomData extends TTokenCustomData, TRegistry extends TAnyRegistryData> = TTokenAuthConfig<TCustomData> & TAuthOutputMetadata<TRegistry, "tokenAuthMetadataSchema">;
type TNoAuth = {
    type: typeof AuthType.NONE;
    config: undefined;
};
type TTokenAuth<TCustomData extends TTokenCustomData, TRegistry extends TAnyRegistryData> = {
    type: typeof AuthType.TOKEN;
    config: TTokenAuthConfigWithOutputMetadata<TCustomData, TRegistry>;
};
type TOAuth<TRegistry extends TAnyRegistryData> = {
    type: typeof AuthType.OAUTH;
    config: TOAuthConfigWithOutputMetadata<TRegistry>;
};
/**
 * The authentication configuration for an action.
 */
type TActionAuth<TRegistry extends TAnyRegistryData, TCustomData extends TTokenCustomData> = TNoAuth | TTokenAuth<TCustomData, TRegistry> | TOAuth<TRegistry>;
type TAnyActionAuth = TActionAuth<TAnyRegistryData, TTokenCustomData>;
/**
 * Generic type for getting the auth argument in the action input function.
 *
 * The auth argument will always have an `accessToken` property, but it may also have a `customData` property if the auth config has a custom data schema.
 */
type TAuthArg<TAuth extends TAnyActionAuth> = TAuth["type"] extends typeof AuthType.NONE ? never : TAuth["type"] extends typeof AuthType.OAUTH ? {
    accessToken: string;
} : TAuth["type"] extends typeof AuthType.TOKEN ? TAuth["config"] extends TTokenAuthConfigWithOutputMetadata<infer TCustomData, TAnyRegistryData> ? TCustomData extends TTokenCustomData ? {
    customData: TCustomData extends z.AnyZodObject ? z.output<TCustomData> : null;
    accessToken: string;
} : never : never : never;

declare class ActionBuilderWithFunction<TLocalActionData extends TAnyActionData> {
    actionData: TLocalActionData;
    constructor({ actionData }: {
        actionData: TLocalActionData;
    });
    setExampleInput(exampleInput: z.input<TLocalActionData["inputSchema"]>): this;
    getId(): TLocalActionData["id"];
    getExampleInput(): TLocalActionData["exampleInput"];
    getNamespace(): TLocalActionData["registryData"]["namespace"];
    getMetadata(): TLocalActionData["metadata"];
    getAuthConfig(): TLocalActionData["authConfig"];
    getInputSchema(): TLocalActionData["inputSchema"];
    getActionFunction(): TLocalActionData["actionFunction"];
    getOutputSchema(): TLocalActionData["outputSchema"];
    getInputJSONSchema(): zod_to_json_schema.JsonSchema7Type & {
        $schema?: string | undefined;
        definitions?: {
            [key: string]: zod_to_json_schema.JsonSchema7Type;
        } | undefined;
    };
    getOutputJSONSchema(): zod_to_json_schema.JsonSchema7Type & {
        $schema?: string | undefined;
        definitions?: {
            [key: string]: zod_to_json_schema.JsonSchema7Type;
        } | undefined;
    };
    getActionType(): TLocalActionData["actionType"];
    getRegistryData(): TLocalActionData["registryData"];
    getFunctionName(): TLocalActionData["functionName"];
    getChatCompletionTool(): OpenAI.Chat.Completions.ChatCompletionTool;
}

type TAnyActionRegistry$1 = Readonly<{
    [key: string]: ActionBuilderWithFunction<any>;
}>;
type TActionCallerInput<TRegistry extends TAnyActionRegistry$1, TId extends keyof TRegistry> = ValuesOf<{
    [K in TId]: {
        name: ReturnType<TRegistry[K]["getFunctionName"]>;
        arguments: z.input<ReturnType<TRegistry[K]["getInputSchema"]>>;
        id?: string;
    };
}>;
type TCallerSuccessResult<TAction extends ActionBuilderWithFunction<any>> = {
    status: "success";
    data: z.output<ReturnType<TAction["getOutputSchema"]>>;
    actionId: ReturnType<TAction["getId"]>;
    functionName: ReturnType<TAction["getFunctionName"]>;
    id: string;
    arguments: z.input<ReturnType<TAction["getInputSchema"]>>;
    parsedArguments: z.output<ReturnType<TAction["getInputSchema"]>>;
    isSuccess: true;
    isError: false;
};
type TCallerErrorResult<TAction extends ActionBuilderWithFunction<any>> = {
    status: "error";
    message: string;
    cause?: Error;
    actionId: ReturnType<TAction["getId"]>;
    data: undefined;
    isError: true;
    isSuccess: false;
    id: string;
    failedArguments: z.input<ReturnType<TAction["getInputSchema"]>>;
    functionName: ReturnType<TAction["getFunctionName"]>;
    failedParsedArguments: z.output<ReturnType<TAction["getInputSchema"]>> | null;
};
type TActionRegistrySubset<TRegistry extends TAnyActionRegistry$1, TSubset extends (keyof TRegistry)[] | undefined = undefined> = TSubset extends (keyof TRegistry)[] ? TSubset[number] : keyof TRegistry;
type TCallerResults<TRegistry extends TAnyActionRegistry$1, TId extends keyof TRegistry> = ValuesOf<{
    [K in TId]: TCallerSuccessResult<TRegistry[K]> | TCallerErrorResult<TRegistry[K]>;
}>[];
type inferExtras<TActionData extends TAnyActionData, T extends Readonly<{
    [K in TActionData["id"]]: ActionBuilderWithFunction<TActionData>;
}>> = ReturnType<ValuesOf<T>["getRegistryData"]> extends infer TRegistry ? TRegistry extends TAnyRegistryData ? TRegistry["actionFunctionExtrasSchema"] extends infer A ? A extends ValidZodSchema ? {
    extras: z.input<A>;
} : {} : {} : {} : {};
type filterByAuthType<T extends TAnyActionRegistry$1, U extends TAuthType, TId extends keyof T, TSuccess> = {
    [K in TId as ReturnType<T[K]["getAuthConfig"]>["type"] extends U ? K : never]: TSuccess;
};
/**
 * A function that fetches the OAuth access token for an action.
 *
 * @param actionId The ID of the action to fetch the OAuth access token for.
 * @returns A promise that resolves to the OAuth access token as { accessToken: string }.
 *
 * @throws An error if the OAuth access token could not be fetched.
 *
 * @example
 * ```ts
 * async fetchOAuthAccessToken(actionId) {
 *    ...fetch the OAuth access token...
 *    return {
 *        accessToken: "the access token"
 *    }
 * }
 *
 */
type TFetchOAuthAccessToken<T> = (actionId: T) => Promise<{
    accessToken: string;
}>;
type inferOAuthFunction<TRegistry extends TAnyActionRegistry$1, TId extends keyof TRegistry> = filterByAuthType<TRegistry, "OAuth", TId, 1> extends infer A ? keyof A extends never ? {} : {
    fetchOAuthAccessToken: TFetchOAuthAccessToken<keyof A>;
} : {};
type TFetchTokenAuthData<T> = (actionId: T) => Promise<{
    accessToken: string;
    customData: z.AnyZodObject["_input"];
}>;
type inferTokenFunction<TRegistry extends TAnyActionRegistry$1, TId extends keyof TRegistry> = filterByAuthType<TRegistry, "Token", TId, 1> extends infer A ? keyof A extends never ? {} : {
    fetchTokenAuthData: TFetchTokenAuthData<keyof A>;
} : {};
type TFunctionCallingArgs<TRegistry extends TAnyActionRegistry$1, U extends (keyof TRegistry)[] | undefined = undefined> = {
    inArray?: U;
    runInParallel?: boolean;
} & inferExtras<TAnyActionData, TRegistry> & inferOAuthFunction<TRegistry, TActionRegistrySubset<TRegistry, U>> & inferTokenFunction<TRegistry, TActionRegistrySubset<TRegistry, U>>;
type TActionCaller<TActionData extends TAnyActionData, TRegistry extends Readonly<{
    [K in TActionData["id"]]: ActionBuilderWithFunction<TActionData>;
}>, U extends (keyof TRegistry)[] | undefined = undefined> = (inputs: TActionCallerInput<TRegistry, TActionRegistrySubset<TRegistry, U>>[]) => Promise<TCallerResults<TRegistry, TActionRegistrySubset<TRegistry, U>>>;
declare const setupActionCaller: <TActionData extends TAnyActionData, const TRegistry extends Readonly<{ [K in TActionData["id"]]: ActionBuilderWithFunction<TActionData>; }>, U extends (keyof TRegistry)[] | undefined = undefined>(registry: TRegistry, args: TFunctionCallingArgs<TRegistry, U>) => {
    /**
     * A function that calls the actions.
     *
     * Returns the results of the actions. Results are in the same order as the inputs and contain the status of the action, the data if the action was successful, and the action ID.
     *
     * @param inputs The inputs to the actions.
     * @returns The results of the actions.
     */
    actionCaller: TActionCaller<TActionData, TRegistry, U>;
};

type TClientMetadata = z.output<ValidZodSchema> | never;
type ClientActionData<TId extends string, TFunctionName extends string, TInputSchema extends TActionInput, TMetadata extends TClientMetadata> = {
    actionId: TId;
    functionName: TFunctionName;
    inputSchema: TInputSchema;
    metadata: TMetadata;
};
type TAnyClientActionRegistry = Readonly<{
    [K in string]: ClientActionData<string, K, TActionInput, TClientMetadata>;
}>;
type inferActionComponentRouter<T extends TAnyClientActionRegistry, Props extends {} = {}> = {
    [K in keyof T]: Props & {
        args: z.output<T[K]["inputSchema"]> | null;
        functionName: T[K]["functionName"];
        inputSchema: T[K]["inputSchema"];
        onSubmit: (args: z.output<T[K]["inputSchema"]>) => void;
        metadata: T[K]["metadata"];
    };
};
type inferActionComponentProps<T extends inferActionComponentRouter<TAnyClientActionRegistry, any>, K extends keyof T> = T[K];
declare const createActionComponentRouter: <TRouter extends inferActionComponentRouter<Readonly<{
    [x: string]: ClientActionData<string, string, TActionInput, TClientMetadata>;
}>, any>>(args: { [K in keyof TRouter]: React.FC<inferActionComponentProps<TRouter, K>>; }) => React.FC<(TRouter extends inferActionComponentRouter<any, infer K_1 extends {}> ? K_1 : {
    other: string;
}) & {
    functionName: string;
    args: unknown;
    inputSchema: TRouter[keyof TRouter]["inputSchema"];
    onSubmit: (args: ValuesOf<{ [K_2 in keyof TRouter]: {
        functionName: K_2;
        args: NonNullable<TRouter[K_2]["args"]>;
    }; }>) => void;
    fallback?: React.ReactNode;
    metadata: TRouter[keyof TRouter]["metadata"];
}>;
type inferClientAct<T extends TAnyActionRegistry, TTransformedMetadata extends TClientMetadata> = {
    [K in keyof T as ReturnType<T[K]["getActionType"]> extends "CLIENT" ? ReturnType<T[K]["getFunctionName"]> : never]: ClientActionData<ReturnType<T[K]["getId"]>, ReturnType<T[K]["getFunctionName"]>, ReturnType<T[K]["getInputSchema"]>, TTransformedMetadata>;
};
declare const createClientActionRegistry: <const T extends Readonly<{
    [key: string]: ActionBuilderWithFunction<any>;
}>, TTransformedMetadata extends TClientMetadata = never>(registry: T, options?: {
    pipeMetadata?: (ReturnType<T[keyof T]["getMetadata"]> extends infer TMetadata ? TMetadata extends any ? (metadata: TMetadata) => TTransformedMetadata : never : never) | undefined;
} | undefined) => inferClientAct<T, TTransformedMetadata>;

/**
 * Filter the action registry by the auth type.
 *
 * @param registry The registry of actions.
 * @param authType The auth type to filter by.
 *
 * @returns The filtered registry of actions.
 */
declare const filterActionRegistryByAuthType: <const T extends Readonly<{
    [key: string]: ActionBuilderWithFunction<any>;
}>, U extends TAuthType>(registry: T, authType: U) => { [K in keyof T as ReturnType<T[K]["getAuthConfig"]>["type"] extends U ? K : never]: T[K]; };
declare const generateActionIdMap: <const T extends Readonly<{
    [key: string]: ActionBuilderWithFunction<any>;
}>>(registry: T) => Readonly<{ [K in keyof T]: K; }>;
/**
 * Filter the action registry by the action type.
 *
 * @param registry The registry of actions.
 * @param authType The auth type to filter by.
 *
 * @returns The filtered registry of actions.
 */
declare const filterActionRegistryByActionType: <const T extends Readonly<{
    [key: string]: ActionBuilderWithFunction<any>;
}>, U extends TActionType>(registry: T, actionType: U) => { [K in keyof T as ReturnType<T[K]["getActionType"]> extends U ? K : never]: T[K]; };
/**
 * Pick the actions from the registry.
 *
 * @param registry The registry of actions.
 * @param id The id of the action to pick.
 *
 * @returns The picked actions.
 *
 */
declare const pickFromActionsRegistry: <T extends Readonly<{
    [key: string]: ActionBuilderWithFunction<any>;
}>, K extends keyof T | (keyof T)[]>(registry: T, id: K) => K extends infer U ? U extends (keyof T)[] ? Pick<T, U[number]> : U extends keyof T ? Pick<T, U> : never : never;
declare const omitFromActionsRegistry: <T extends Readonly<{
    [key: string]: ActionBuilderWithFunction<any>;
}>, K extends keyof T | (keyof T)[]>(registry: T, id: K) => K extends infer U ? U extends (keyof T)[] ? Omit<T, U[number]> : U extends keyof T ? Omit<T, U> : never : never;

type TActionBuilderWithAuthData<TOutputActionData extends TActionDataWithOutput, TAuth extends TActionAuth<TOutputActionData["registryData"], TTokenCustomData>> = TOutputActionData & {
    authConfig: TAuth;
};
type TActionDataWithAuth = TActionBuilderWithAuthData<TActionDataWithOutput, TAnyActionAuth>;
declare class ActionBuilderWithAuth<TLocalActionData extends TActionDataWithAuth> {
    actionData: TLocalActionData;
    constructor({ actionData }: {
        actionData: TLocalActionData;
    });
    /**
     * The function that gets called when the action is invoked. This function should return a promise that resolves to the output of the action.
     *
     * @param input The input to the action function. This is the input that the user provides when invoking the action.
     * @param auth The authentication information for the action. This is the authentication information that the user provides when invoking the action.
     *
     * @returns A promise that resolves to the output of the action.
     *
     * @example
     * The following action function is an example for a `send-email` action.
     * ```
     * async ({ input, auth }) => {
     *   const { subject, body, to } = input;
     *   const { accessToken } = auth;
     *   ...code to send email using the input...
     *   return;
     * }
     * ```
     *
     * @example
     * The following action function is an example for a `get-google-contact` action.
     * ```
     * async ({ input, auth }) => {
     *   const contact = ...code to get contact...
     *   return contact;
     * }
     * ```
     */
    setActionFunction(actionFunction: TActionFunction<TLocalActionData["registryData"], TLocalActionData["inputSchema"], TLocalActionData["outputSchema"], TLocalActionData["authConfig"]>): ActionBuilderWithFunction<TActionData<TLocalActionData["registryData"], TLocalActionData["id"], TLocalActionData["functionName"], TLocalActionData["inputSchema"], TLocalActionData["outputSchema"], TLocalActionData["authConfig"], TLocalActionData["actionType"]>>;
}

declare class ActionBuilderWithOAuthType<TLocalActionData extends TActionDataWithOutput> {
    actionData: TLocalActionData;
    constructor({ actionData }: {
        actionData: TLocalActionData;
    });
    setOAuthData: (data: TOAuthConfigWithInputMetadata<TLocalActionData["registryData"]>) => ActionBuilderWithAuth<TLocalActionData & {
        authConfig: {
            type: "OAuth";
            config: TOAuthConfigWithOutputMetadata<TLocalActionData["registryData"]>;
        };
    }>;
}

declare class ActionBuilderWithTokenType<TLocalActionData extends TActionDataWithOutput> {
    actionData: TLocalActionData;
    constructor({ actionData }: {
        actionData: TLocalActionData;
    });
    setTokenData: <T extends TTokenCustomData>(data: TTokenAuthConfigWithInputMetadata<T, TLocalActionData["registryData"]>) => ActionBuilderWithAuth<TLocalActionData & {
        authConfig: TTokenAuth<T, TLocalActionData["registryData"]>;
    }>;
}

type TActionBuilderWithOutputData<TInputActionData extends TActionDataWithInput, TOutput extends TActionOutput, TType extends TActionType> = TInputActionData & {
    outputSchema: TOutput;
    actionType: TType;
};
type TActionDataWithOutput = TActionBuilderWithOutputData<TActionDataWithInput, TActionOutput, TActionType>;
declare class ActionBuilderWithOutput<TLocalActionData extends TActionDataWithOutput> {
    actionData: TLocalActionData;
    constructor({ actionData }: {
        actionData: TLocalActionData;
    });
    /**
     * The authentication type for the action. This is used to generate the authentication page
     * for the action and get users authenticated.
     */
    setAuthType(type: typeof AuthType.NONE): ActionBuilderWithAuth<TActionBuilderWithAuthData<TLocalActionData, TNoAuth>>;
    setAuthType(type: typeof AuthType.TOKEN): ActionBuilderWithTokenType<TLocalActionData>;
    setAuthType(type: typeof AuthType.OAUTH): ActionBuilderWithOAuthType<TLocalActionData>;
}

declare class ActionBuilderWithActionType<TType extends TActionType, TLocalActionData extends TActionDataWithInput> {
    actionData: TLocalActionData;
    actionType: TType;
    constructor({ actionData, actionType, }: {
        actionData: TLocalActionData;
        actionType: TType;
    });
    /**
     * `JSON Data` that adheres to a specific Zod schema. This is typically the output for actions that `GET data`. If this is the case, you should specify the Zod schema for the output.
     */
    setOutputSchema<T extends TActionOutput>(output: T): ActionBuilderWithOutput<TLocalActionData & {
        outputSchema: T;
        actionType: TType;
    }>;
    setOutputAsVoid(): ActionBuilderWithOutput<TLocalActionData & {
        actionType: TType;
        outputSchema: z.ZodVoid;
    }>;
    setOutputSchemaAsInputSchema(): ActionBuilderWithOutput<TActionBuilderWithOutputData<TLocalActionData, TLocalActionData["inputSchema"], TType>>;
}

type TActionBuilderWithInputData<TBuilderData extends TActionBuilderData, TInput extends TActionInput> = TBuilderData & Pick<TActionData<any, any, any, TInput, any, any, any>, "inputSchema">;
type TActionDataWithInput = TActionBuilderWithInputData<TActionBuilderData, TActionInput>;
type TEcho<TLocalActionData extends TActionDataWithInput> = TLocalActionData["inputSchema"] extends infer TOutput ? TOutput extends TActionOutput ? ReturnType<ActionBuilderWithAuth<TActionBuilderWithAuthData<TLocalActionData & {
    outputSchema: TOutput;
    actionType: "ECHO";
}, TNoAuth>>["setActionFunction"]> : never : never;
declare class ActionBuilderWithInput<TLocalActionData extends TActionDataWithInput> {
    actionData: TLocalActionData;
    constructor({ actionData }: {
        actionData: TLocalActionData;
    });
    /**
     * Spark has three options for actions:
     *
     * 1.`JSON Data` that adheres to a specific Zod schema. This is typically the output for actions that `GET data`. If this is the case, you should specify the Zod schema for the output.
     *
     * 2.`React Component` that asks for confirmation for the action. This is typically the output for actions that `POST data`. If this is the case, you should specify a React component that asks for confirmation for the action. The output will be `void`.
     *
     * 3. `Echo` that just simply returns the input data.
     */
    setActionType<T extends TActionType>(type: T): ActionBuilderWithActionType<T, TLocalActionData>;
    setActionType(type: "ECHO"): TEcho<TLocalActionData>;
}

type TActionBuilderConstructorData<TRegistry extends TAnyRegistryData, TId extends string, TFunctionName extends string> = Pick<TActionData<TRegistry, TId, TFunctionName, any, any, any, any>, "metadata" | "id" | "registryData" | "functionName">;
type TActionBuilderData = TActionBuilderConstructorData<TAnyRegistryData, string, string>;
declare class ActionBuilder<TLocalActionData extends TActionBuilderData> {
    actionData: TLocalActionData;
    constructor(input: TLocalActionData);
    /**
     * The Zod input schema for the action. This is used to validate the arguments passed to the action.
     *
     * You must include a `.describe()` call on each field to provide a description of the input. This description will be used to generate the prompts and details page for your action.
     *
     * @example
     * The following input schema is an example for a `send-email` action.
     * ```
     * z.object({
     *  subject: z.string().desribe("The subject of the email"),
     *  body: z.string().describe("The body of the email"),
     *  to: z.array(z.object({
     *   email: z.string().describe("The email of the recipient"),
     * })).describe("The recipients of the email"),
     * ```
     */
    setInputSchema<T extends TActionInput>(input: T): ActionBuilderWithInput<TLocalActionData & {
        inputSchema: T;
    }>;
}

type TActionsArray<TRegistry extends TAnyRegistryData> = Array<ReturnType<ActionBuilderWithFunction<any>["getRegistryData"]> extends TRegistry ? ActionBuilderWithFunction<any> : never>;
type TActionsRegistry<TRegistry extends TAnyRegistryData, T extends TActionsArray<TRegistry>> = T extends [infer A, ...infer R] ? A extends ActionBuilderWithFunction<any> ? ReturnType<A["getRegistryData"]> extends infer ARegistryData ? ARegistryData extends TRegistry ? ReturnType<A["getId"]> extends infer TId ? TId extends string ? R extends TActionsArray<TRegistry> ? Readonly<{
    [K in TId]: A;
}> & TActionsRegistry<TRegistry, R> : never : never : never : never : never : never : Readonly<{}>;
type TAnyActionRegistry = Readonly<{
    [key: string]: ActionBuilderWithFunction<any>;
}>;
type TCreateActionsRegistry<T extends string> = `create${T}ActionsRegistry`;
type TCreateAction<T extends string> = `create${T}Action`;
type TCreateActionsRegistryFunction<TRegistry extends TAnyRegistryData> = <const T extends TActionsArray<TRegistry>>(registry: T) => TActionsRegistry<TRegistry, T>;
type TCreateActionFunction<TRegistry extends TAnyRegistryData> = TRegistry["metadataSchema"] extends infer TMetadataSchema ? TMetadataSchema extends TActionMetadata | unknown ? <TId extends string, TFunctionName extends string = TId>(input: {
    id: TId;
    functionName?: TFunctionName;
} & (TMetadataSchema extends ValidZodSchema ? {
    metadata: z.input<TMetadataSchema>;
} : {})) => ActionBuilder<TActionBuilderConstructorData<TRegistry, TId, TFunctionName>> : never : never;
type TGenerateFunctionsRet<TRegistry extends TAnyRegistryData> = TRegistry["namespace"] extends infer TNamespace ? TNamespace extends string ? Readonly<{
    [K in TCreateActionsRegistry<TNamespace>]: TCreateActionsRegistryFunction<TRegistry>;
} & {
    [K in TCreateAction<TNamespace>]: TCreateActionFunction<TRegistry>;
}> : never : never;
/**
 * Generate the functions to create an action registry.
 *
 * @param args The arguments to generate the functions.
 * @returns The functions to create an action registry.
 *
 * @example
 * ```
 * const { createADEActionsRegistry, createADEAction } = generateActionRegistryFunctions({
 *   namespace: "ADE",
 *   metadataSchema: z.object({
 *     title: z.string(),
 *     description: z.string(),
 *   }),
 *   actionFunctionExtras: z.object({
 *     userData: z.object({
 *       email: z.string().email(),
 *       name: z.string(),
 *     })
 *   }),
 * })
 * ```
 */
declare const generateActionRegistryFunctions: <const TRegistry extends TAnyRegistryData>(args: TRegistry) => TGenerateFunctionsRet<TRegistry>;
declare const createActionsRegistry: TCreateActionsRegistryFunction<{
    readonly namespace: "";
    readonly metadataSchema: undefined;
    readonly actionFunctionExtrasSchema: undefined;
    readonly tokenAuthMetadataSchema: undefined;
    readonly oAuthMetadataSchema: undefined;
}>;
declare const createAction: <TId extends string, TFunctionName extends string = TId>(input: {
    id: TId;
    functionName?: TFunctionName | undefined;
}) => ActionBuilder<TActionBuilderConstructorData<{
    readonly namespace: "";
    readonly metadataSchema: undefined;
    readonly actionFunctionExtrasSchema: undefined;
    readonly tokenAuthMetadataSchema: undefined;
    readonly oAuthMetadataSchema: undefined;
}, TId, TFunctionName>>;
type inferAcitonMetadataSchema<T> = T extends TCreateActionFunction<infer TRegistry> ? TRegistry["metadataSchema"] extends infer TMetadata ? TMetadata extends TActionMetadata ? TMetadata : never : never : never;

/**
 * Given an action registry, generate the tools for the LLM.
 *
 * @param registry The registry of actions.
 * @param toolId The id(s) of the action(s) to generate tools for. Can be a single id or an array of ids.
 *
 * @returns The tools for the LLM. In the format of `OpenAI.Chat.Completions.ChatCompletionTool[]`. The name of the tool is the ID of the action.
 *
 * @example
 */
declare const generateLLMTools: <const T extends Readonly<{
    [key: string]: ActionBuilderWithFunction<any>;
}>, U extends (keyof T)[] | undefined>(registry: T, args?: {
    inArray?: U | undefined;
} | undefined) => OpenAI.Chat.Completions.ChatCompletionTool[];
type TToolCallHandler<T extends TAnyActionRegistry, U extends (keyof T)[] | undefined> = (toolCalls: OpenAI.Chat.Completions.ChatCompletionMessageToolCall[]) => Promise<{
    toolCallMessages: OpenAI.Chat.Completions.ChatCompletionToolMessageParam[];
    results: TCallerResults<T, TActionRegistrySubset<T, U>>;
}>;
type TCreateFewShotToolCallMessages<TRegistry extends TAnyActionRegistry, U extends (keyof TRegistry)[] | undefined> = (examples: {
    userMessageContent: string;
    tool_calls: ValuesOf<{
        [K in TActionRegistrySubset<TRegistry, U>]: {
            name: ReturnType<TRegistry[K]["getFunctionName"]>;
            arguments: z.input<ReturnType<TRegistry[K]["getInputSchema"]>>;
        } & (ReturnType<TRegistry[K]["getActionType"]> extends "ECHO" ? {} : ReturnType<TRegistry[K]["getOutputSchema"]> extends z.ZodVoid ? {} : {
            response: z.output<ReturnType<TRegistry[K]["getOutputSchema"]>>;
        });
    }>[];
}[]) => OpenAI.Chat.Completions.ChatCompletionMessageParam[];
type TChooseTool<TRegistry extends TAnyActionRegistry, U extends (keyof TRegistry)[] | undefined = undefined> = (name: ReturnType<TRegistry[TActionRegistrySubset<TRegistry, U>]["getFunctionName"]>) => OpenAI.Chat.Completions.ChatCompletionToolChoiceOption;
declare const setupFunctionCalling: <TActionData extends TAnyActionData, const T extends Readonly<{ [K in TActionData["id"]]: ActionBuilderWithFunction<TActionData>; }>, U extends (keyof T)[] | undefined = undefined>(registry: T, args: {
    inArray?: U | undefined;
    runInParallel?: boolean | undefined;
} & (ReturnType<ValuesOf<T>["getRegistryData"]> extends infer TRegistry ? TRegistry extends TAnyRegistryData ? TRegistry["actionFunctionExtrasSchema"] extends infer A ? A extends ValidZodSchema ? {
    extras: z.input<A>;
} : {} : {} : {} : {}) & ((TActionRegistrySubset<T, U> extends infer T_1 extends keyof T ? { [K in T_1 as ReturnType<T[K]["getAuthConfig"]>["type"] extends "OAuth" ? K : never]: 1; } : never) extends infer A_1 ? keyof A_1 extends never ? {} : {
    fetchOAuthAccessToken: (actionId: keyof A_1) => Promise<{
        accessToken: string;
    }>;
} : {}) & ((TActionRegistrySubset<T, U> extends infer T_2 extends keyof T ? { [K_1 in T_2 as ReturnType<T[K_1]["getAuthConfig"]>["type"] extends "Token" ? K_1 : never]: 1; } : never) extends infer A_2 ? keyof A_2 extends never ? {} : {
    fetchTokenAuthData: (actionId: keyof A_2) => Promise<{
        accessToken: string;
        customData: {
            [x: string]: any;
        };
    }>;
} : {}) & {
    /**
     * Whether to include errors in the tool call messages.
     */
    includeErrorsInToolCallMessages?: boolean | undefined;
}) => {
    tools: OpenAI.Chat.Completions.ChatCompletionTool[];
    toolCallsHandler: TToolCallHandler<T, U>;
    chooseTool: TChooseTool<T, U>;
    createFewShotToolCallMessages: TCreateFewShotToolCallMessages<T, U>;
};

export { AuthType, type FilterObjectByValue, type TActionCallerInput, type TActionInput, type TActionMetadata, type TActionRegistrySubset, type TAnyActionRegistry, type TAnyClientActionRegistry, type TCallerErrorResult, type TCallerResults, type TCallerSuccessResult, type TFunctionCallingArgs, type ValuesOf, createAction, createActionComponentRouter, createActionsRegistry, createClientActionRegistry, filterActionRegistryByActionType, filterActionRegistryByAuthType, generateActionIdMap, generateActionRegistryFunctions, generateLLMTools, type inferAcitonMetadataSchema, type inferActionComponentProps, type inferActionComponentRouter, omitFromActionsRegistry, pickFromActionsRegistry, setupActionCaller, setupFunctionCalling };
